function [p, t,skelp,J_Axis] = drawMesh(mesh, h_a, parameter)
%% This function draw the mesh
% DRAWMESH Draw a mesh including the skeletton if available
% function [p, t,skelp,J_Axis] = drawMesh(mesh, h_a, parameter)
% INPUT
%  mesh
%   h_a (optional, axes handle into which the mesh should be drawn.)
%   parameter
% OUTPUT
%     p: Mesh patch plot handle
%     t: trace vertices handle
%     skelp: skletton handle
%     JAxis: Joint axis plot handle
%
% This function is part of the MPI08 data set, to be obtained at 
% http://www.tnt.uni-hannover.de/project/MPI08_Database/
%
%
% Written by: Gerard Pons (modified from Andreas Baak)
% Revision:   1.0
% Date:       20.01.2010
%
% Copyright 2010, MPI Informatik, Germany, and Hannover University, Germany.
% All Rights Reserved Permission to use, copy, modify, and distribute this 
% software and its documentation for any non-commercial purpose is hereby 
% granted without fee, provided that the above copyright notice appear in 
% all copies and that both that copyright notice and this permission 
% notice appear in supporting documentation, and that the name of the 
% author not be used in advertising or publicity pertaining to distribution 
% of the software without specific, written prior permission. 
% THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY 
% PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
% SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER 
% RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF 
% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
% CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 

parameterDefault.clickableVertices = false;
global ctrl

if ~exist('parameter', 'var')
    parameter = struct();
    parameter.clickableVertices=false;
end
%parameter = mergeParameters(parameter, parameterDefault);
vertices = mesh.vertices';
faces = mesh.faces';

if isfield(mesh, 'traceVertices')
    traceVertices = mesh.traceVertices;
else
    traceVertices =[];
end
if ~exist('h_a', 'var')
    h_f = gcf;
    h_a=gca;
end
set(h_a, 'visible', 'off');
set(h_a, 'projection', 'perspective');

%  draw mesh
if (1)
    p=patch('Parent',h_a,'Faces', faces, 'Vertices', vertices, 'FaceVertexCData', repmat([1 1 0], size(vertices, 1), 1), 'FaceColor', 'interp', 'EdgeColor', 'interp');
else
    %To draw vertices colored according to Joint influence in the kinematic chain%
    JColor=load('JointColorTable.mat');
    JointColor=JColor.JointColorBall;
    sk=mesh.skinningJoints(1,:)';
    RGBtable=cell(numel(sk),1);
    [RGBtable{1:end}]=JointColor{sk,1};
    RGBtable=cell2mat(RGBtable);
    p=patch('Parent',h_a,'Faces', faces, 'Vertices', vertices, 'FaceVertexCData', RGBtable, 'FaceColor', 'interp', 'EdgeColor', 'interp');
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end
axis equal;
%axis tight;
axis off
set(p, 'faceAlpha', 0.9);
set(p, 'edgeAlpha', 0.2);
%set(p, 'marker', '.', 'markersize', 3, 'markeredgecolor', 'k');
set(p, 'ambientStrength', 0.4);
set(p, 'diffuseStrength', 0.4);
set(p, 'BackFaceLighting', 'unlit')
set(p, 'FaceLighting', 'phong');
set(p, 'edgeLighting', 'phong');
hold on;
if parameter.clickableVertices
    for v = 1 : size(vertices, 1);
        h = plot3(vertices(v, 1), vertices(v, 2), vertices(v, 3), '.', 'markersize', 15, 'markeredgecolor', [0.3 0.3 0.3]);
        set(h, 'buttondownfcn', {@printVertexNumber, v});
    end
end
% Draw trace vertices
t=plot3(vertices(traceVertices, 1), vertices(traceVertices, 2), vertices(traceVertices, 3), '.', 'markersize', 10, 'markeredgecolor', 'r');
%  Draw skeleton
if (isfield(mesh, 'skel'))
    % draw a skeleton inside
    nodes = mesh.skel.nodes;
    [skelVertices, skelFaces] = generateSkeletonPatch(nodes);
    numVertices = size(skelVertices, 2);
    skelp=patch('Parent',h_a,'Faces', skelFaces', 'Vertices', skelVertices', 'FaceVertexCData', repmat([1 0 0],numVertices, 1), 'FaceColor', 'flat', 'EdgeColor', 'flat');
    set(skelp, 'marker', '.', 'markersize', 25);
    set(skelp, 'edgeLighting', 'none');
    set(skelp, 'edgeAlpha', 0.8);  %0.3
    set(skelp, 'faceLighting', 'none');
    set(skelp, 'lineWidth', 3);
    if 0
        J_Axis=drawJointAxis(mesh.skel);
    else
        J_Axis=-1;
    end
else
    skelp=-1;
    J_Axis=-1;
end
% Set CAMERA PARAMS
drawCoordinateSystem(eye(3),[0 0 0],0.2);
% set up light
set(h_a, 'ambientLightColor', [1 1 1]);
delete(findobj(h_a, 'type', 'light'));
camlight;
camlight head
camlight left;
set(h_a, 'cameraupvector', [0 1 0]);
cameratoolbar('setmode', 'orbit');
cameratoolbar('setcoordsys', 'y');
cameratoolbar('show');
set(h_a, 'visible', 'on');
%set(h_f,'Color','black')
%%%% To make it cool %%%%%%%%%%%%
% axis off
%set(gcf,'color','k');
set(p,'FaceColor',[0.7,0.78,1],'EdgeColor','none')
set(p,'FaceColor',[0.04,0.52,0.78],'EdgeColor','none')
set(p,'EdgeColor','none')
set(p,'EdgeColor','none')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function printVertexNumber(src, evnt, v)
        fprintf('My vertex ID is %d.\n', v);
    end
end
